<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Controllers;

use MicroEngine\Admin\Forms\ConfigForm;
use Psr\Http\Message\ResponseInterface;
use Interop\Container\ContainerInterface;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;

class ConfigController extends Controller
{
    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        parent::__construct($c);

        $this->c->userManager->required('config');
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $configs = config('settings.config_files');

        return $this->view->render(
            $response,
            'admin/views/config/index.twig',
            compact('configs')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new ConfigForm();
        $filename = $args['filename'];
        $configService = $this->c->configService;

        // 編集が許可されたファイルであることを確認
        if (!$configService->isEditable($filename)) {
            $this->flash->addMessage('error', '指定されたファイルを開くことができません。');
            return $response->withRedirect($this->c->router->pathFor('admin.config.index'));
        }

        $content = $configService->getContent($filename);
        $form->setInputValue([
            'filename' => $filename,
            'content' => $content,
        ]);
        $label = $configService->getLabel($filename);
        $codeEditor = $this->c->codeEditor;
        $mode = $codeEditor->getMode($filename);

        return $this->view->render(
            $response,
            'admin/views/config/edit.twig',
            compact('form', 'label', 'mode')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new ConfigForm();
        $filename = $args['filename'];
        $configService = $this->c->configService;

        // 編集が許可されたファイルであることを確認
        if (!$configService->isEditable($filename)) {
            $this->flash->addMessage('error', '指定されたファイルを開くことができません。');
            return $response->withRedirect($this->c->router->pathFor('admin.config.index'));
        }

        $form->setInputValue($request->getParsedBody());
        $label = $configService->getLabel($filename);

        // バリデーション
        if ($form->validate()) {
            $configService->saveFile($filename, $form->get('content'));
            $this->flash->addMessage('success', '設定ファイルを保存しました。ファイル：' . $label . ' ( ' . $filename . ' )');

            return $response->withRedirect($this->c->router->pathFor('admin.config.index'));
        } else {
            $codeEditor = $this->c->codeEditor;
            $mode = $codeEditor->getMode($filename);

            return $this->view->render(
                $response,
                'admin/views/config/edit.twig',
                compact('form', 'label', 'mode')
            );
        }
    }
}
