<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Controllers;

use Psr\Http\Message\ResponseInterface;
use MicroEngine\Admin\Forms\LicenseForm;
use Interop\Container\ContainerInterface;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;

class LicenseController extends Controller
{
    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        parent::__construct($c);

        $this->c->userManager->required('license');
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // ライセンス情報取得
        $info = $this->c->license->info();

        return $this->view->render(
            $response,
            'admin/views/license/index.twig',
            compact('info')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function register(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new LicenseForm($this->c);

        return $this->view->render(
            $response,
            'admin/views/license/register.twig',
            compact('form')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function store(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new LicenseForm($this->c);
        $form->setInputValue($request->getUploadedFiles());

        // バリデーション
        if ($form->validate()) {
            // ライセンスファイルを保存する
            $this->c->license->store($form->get('license'));
            $this->flash->addMessage('success', 'ライセンスを登録しました。');

            return $response->withRedirect($this->c->router->pathFor('admin.license.index'));
        } else {
            return $this->view->render(
                $response,
                'admin/views/license/register.twig',
                compact('form')
            );
        }
    }
}
