<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Views;

use Twig_Function;
use Twig_Extension;
use Interop\Container\ContainerInterface;

class AdminExtension extends Twig_Extension
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var \MicroEngine\Admin\Auth\Auth
     */
    protected $auth;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->auth = $c->auth;
    }

    /**
     * Twig function 定義
     *
     * @return array
     */
    public function getFunctions(): array
    {
        return [
            new Twig_Function('auth', function () {
                if ($this->auth->authenticated()) {
                    return $this->auth->getUser();
                } else {
                    return false;
                }
            }),
            new Twig_Function('portlet', function ($name) {
                $portlet = $this->c->portlet;

                return $portlet->portlet($name);
            }, ['is_safe' => ['html']]),
            new Twig_Function('current_path', function () {
                return $this->c->request->getUri()->getPath();
            }),
        ];
    }
}
