<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Archive;

use Carbon\Carbon;
use MicroEngine\Core\Support\TermLink;
use Illuminate\Database\Eloquent\Builder;

trait ArchiveTrait
{
    /**
     * 期間リスト取得スコープ
     *
     * @param Builder $query
     * @return mixed
     */
    public function scopeTermList(Builder $query)
    {
        return $query->selectRaw("date_format(date, '%Y') as year, date_format(date, '%m') as month")
            ->orderBy('year', 'asc')
            ->orderBy('month', 'asc')
            ->distinct();
    }

    /**
     * 月間アーカイブスコープ
     *
     * @param Builder $query
     * @param array $options
     * @param string|null $year
     * @param string|null $month
     * @return MonthlyArchive
     */
    public function scopeMonthlyArchive(Builder $query, array $options, string $year = null, string $month = null): MonthlyArchive
    {
        // 設定の初期値をセット
        $options += [
            'month' => 1,
            'offset_month' => 0,
            'sort' => 'asc',
            'blank' => 0,
            'link_format' => 'Y年m月',
        ];

        // 期間リストを取得
        $termQuery = clone $query;
        $termList = $termQuery->termList()->get();

        // リンクリストを取得
        $termLink = new TermLink();
        $termLink->setTerm($termList, $options);

        // indexページの場合は $year, $month はわたされないので、リンク情報取得時に判定した年月をセットする
        if ($year === null) {
            $year = $termLink->getYear();
        }
        if ($month === null) {
            $month = $termLink->getMonth();
        }

        // その年月の１日に日時を作成する
        $dt = Carbon::create($year, $month, 1, 0, 0, 0);

        // 該当期間の開始日時
        $fromDate = $dt->format('Y-m-d');
        // 該当期間の終了日時
        $toDate = $dt->addMonth($options['month'])->format('Y-m-d');

        // 該当期間の情報を取得する
        $items = $query
            ->where('date', '>=', $fromDate)
            ->where('date', '<', $toDate)
            ->orderBy('date', $options['sort'])
            ->get();

        // アーカイブオブジェクトを作成する
        return new MonthlyArchive($items, $termLink->getLinks(), $options, $year, $month);
    }
}
