<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Models;

use MicroEngine\Admin\Models\User;
use Illuminate\Database\Eloquent\Model;
use MicroEngine\Cms\Archive\ArchiveTrait;
use MicroEngine\Cms\Scopes\PublishedScope;
use MicroEngine\Cms\Calendar\CalendarTrait;
use MicroEngine\Core\Pagination\PaginationTrait;

class Calendar extends Model
{
    use PublishTrait, PaginationTrait, ArchiveTrait, CalendarTrait;

    /**
     * @var array
     */
    protected $fillable = ['date', 'type', 'fields', 'visibility', 'published_at', 'closed_at', 'create_user_id', 'update_user_id'];
    /**
     * @var array
     */
    protected $casts = ['fields' => 'array'];
    /**
     * @var string
     */
    protected $baseType = 'calendar';

    /**
     * モデルの「初期起動」メソッド
     *
     * @return void
     */
    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope(new PublishedScope());
    }

    /**
     * 作成ユーザー
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function createUser()
    {
        return $this->belongsTo(User::class, 'create_user_id');
    }

    /**
     * 更新ユーザー
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function updateUser()
    {
        return $this->belongsTo(User::class, 'update_user_id');
    }

    /**
     * @return string
     */
    public function getBaseType(): string
    {
        return $this->baseType;
    }
}
