<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Models;

use MicroEngine\Admin\Models\User;
use Illuminate\Database\Eloquent\Model;

class file extends Model
{
    /**
     * @var array
     */
    protected $fillable = ['is_dir', 'dir', 'filename', 'extension', 'size', 'name', 'create_user_id', 'update_user_id'];

    /**
     * 作成ユーザー
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function createUser()
    {
        return $this->belongsTo(User::class, 'create_user_id');
    }

    /**
     * 更新ユーザー
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function updateUser()
    {
        return $this->belongsTo(User::class, 'update_user_id');
    }

    /**
     * パスを返す
     * 例： img/flower.jpg
     *
     * @return string
     */
    public function path(): string
    {
        $path = $this->dir . $this->filename;
        if ($this->is_dir) {
            $path .= '/';
        }
        return $path;
    }

    /**
     * 拡張子から画像かどうか判定する
     *
     * @return bool
     */
    public function isImg(): bool
    {
        $formats = config('image.extensions');

        return in_array($this->extension, $formats);
    }

    /**
     * href属性用のパスを返す
     *
     * @return string
     */
    public function href(): string
    {
        return '/' . config('cms.files_dir', 'files') . $this->path();
    }

    /**
     * サムネイルのパスを返す
     *
     * @param string $option
     * @return string
     */
    public function thumbSrc(string $option): string
    {
        $filesPath = '/' . config('cms.files_dir', 'files');

        return $filesPath . resize($this->path(), $option);
    }
}
