<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Models;

use MicroEngine\Admin\Models\User;
use Illuminate\Database\Eloquent\Model;
use MicroEngine\Cms\Archive\ArchiveTrait;
use MicroEngine\Cms\Scopes\PublishedScope;
use MicroEngine\Core\Pagination\PaginationTrait;

class Page extends Model
{
    use PublishTrait, PaginationTrait, ArchiveTrait;

    /**
     * @var array
     */
    protected $fillable = ['draft_id', 'type', 'template', 'category_id', 'filename', 'path', 'name', 'fields', 'display_order', 'sitemap', 'date', 'visibility', 'published_at', 'closed_at', 'create_user_id', 'update_user_id'];
    /**
     * @var array
     */
    protected $casts = ['fields' => 'array'];
    /**
     * @var string
     */
    protected $baseType = 'page';

    /**
     * モデルの「初期起動」メソッド
     *
     * @return void
     */
    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope(new PublishedScope());
    }

    /**
     * １対多リレーション
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function metas()
    {
        return $this->hasMany(PageMeta::class);
    }

    /**
     * １対１リレーション
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasOne
     */
    public function search()
    {
        return $this->hasOne(PageSearch::class);
    }

    /**
     * 作成ユーザー
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function createUser()
    {
        return $this->belongsTo(User::class, 'create_user_id');
    }

    /**
     * 更新ユーザー
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function updateUser()
    {
        return $this->belongsTo(User::class, 'update_user_id');
    }

    /**
     * @return string
     */
    public function getBaseType()
    {
        return $this->baseType;
    }

    /**
     * テンプレートパスを取得
     *
     * @return string
     */
    public function getTemplatePath()
    {
        $config = config('pages.' . $this->type);
        if (isset($config['templates'])) {
            $template = '@site/page/' . $this->template;
        } else {
            $template = '@site/page/' . $this->type . '.twig';
        }
        return $template;
    }
}
