<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Models;

use MicroEngine\Admin\Models\User;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;

class Table extends Model
{
    /**
     * @var array
     */
    protected $fillable = ['data_type', 'type', 'data', 'line_count', 'create_user_id', 'update_user_id'];
    /**
     * @var array
     */
    protected $casts = ['data' => 'array'];

    /**
     * 作成ユーザー
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function createUser()
    {
        return $this->belongsTo(User::class, 'create_user_id');
    }

    /**
     * 更新ユーザー
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function updateUser()
    {
        return $this->belongsTo(User::class, 'update_user_id');
    }

    /**
     * JSON文字列を返す
     *
     * @param array $config
     * @return string
     */
    public function jsonData(array $config): string
    {
        if ($config['data_type'] === 'matrix') {
            // matrix タイプの場合
            // 配列の配列に変換してからJSON化する
            $wrappedData = [];
            $columnKeys = array_keys($config['columns']);
            foreach ($this->data as $row) {
                $wrappedRow = [];
                foreach ($columnKeys as $key) {
                    if (isset($row[$key])) {
                        $wrappedRow[] = $row[$key];
                    } else {
                        // 設定変更で追加された列にはnullをセットする
                        $wrappedRow[] = null;
                    }
                }
                $wrappedData[] = $wrappedRow;
            }
            $json = json_encode($wrappedData);
        } else {
            // list, hash はJSオブジェクトのキー順が保証されない仕様に対応するために、配列でラップする。
            $wrappedData = [];

            foreach ($this->data as $key => $value) {
                $array = [];
                // 並びを制御するために、keyを先にセットする。
                if ($config['data_type'] === 'hash') {
                    $array[] = $key;
                }
                $array[] = $value;
                $wrappedData[] = $array;
            }
            $json = json_encode($wrappedData);
        }
        return $json;
    }

    /**
     * 指定したタイプのテーブルコンテンツを取得する
     *
     * @param Builder $query
     * @param string $type
     * @return array
     */
    public function scopeType(Builder $query, string $type = ''): array
    {
        $table = $query->where('type', $type)->first();
        if (!$table) {
            return [];
        }
        return $table->data;
    }
}
