<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Scopes;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Scope;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;
use MicroEngine\Cms\Services\ContentService;

class PublishedScope implements Scope
{
    /**
     * 公開状態のコンテンツに絞り込む
     *
     * @param  \Illuminate\Database\Eloquent\Builder $builder
     * @param  \Illuminate\Database\Eloquent\Model $model
     * @return void
     */
    public function apply(Builder $builder, Model $model)
    {
        $builder->where('visibility', ContentService::VISIBILITY_PUBLIC)
            ->where(function ($query) {
                // 0 で ゼロ値（'0000-00-00 00:00:00'）と一致させる
                //$query->where('published_at', 0)
                $query->where('published_at', null)
                    ->orWhere('published_at', '<=', Carbon::now());
            })
            ->where(function ($query) {
                // 0 で ゼロ値（'0000-00-00 00:00:00'）と一致させる
                //$query->where('closed_at', 0)
                $query->where('closed_at', null)
                    ->orWhere('closed_at', '>', Carbon::now());
            });
    }
}
