<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Views;

use Twig_Function;
use Twig_Extension;
use MicroEngine\Cms\Models\Table;
use Interop\Container\ContainerInterface;
use MicroEngine\Cms\Services\ContentService;

class CmsExtension extends Twig_Extension
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var ContentService
     */
    protected $contentService;

    /**
     * CmsExtension constructor.
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->contentService = $c->contentService;
    }

    /**
     * 関数登録
     * @return array
     */
    public function getFunctions(): array
    {
        return [
            new Twig_Function('input_template_path', function ($field = '', $config = []) {
                if (!is_array($config)) {
                    return '';
                }
                return $this->contentService->inputTemplatePath($field, $config);
            }),
            new Twig_Function('edit_with_code_editor', function ($path) {
                $codeEditor = $this->c->codeEditor;

                return $codeEditor->isEditable($path);
            }),
            new Twig_Function('table', function ($type = '') {
                return Table::type($type);
            }),
        ];
    }
}
