<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Support;

/**
 * ブラウザのタイムアウトを防ぐためにレスポンスをある程度定期的に返すための機能
 */
class ConnectionKeeper
{
    /**
     * @var 前回のフラッシュ時のunix timestamp
     */
    protected $flushedTs;

    /**
     * 初期化処理
     *
     * @param string|null $html
     */
    public function init(string $html = null)
    {
        // PHPの実行時間を1時間まで延長する
        set_time_limit(3600);

        if ($html === null) {
            $html = <<<EOD
<!DOCTYPE html>
<html lang="ja">
<head>
<meta charset="UTF-8">
<title></title>
</head>
<body>
EOD;
        }
        $this->flush($html);
    }

    /**
     * コネクションをキープするためにレスポンスを返す
     *
     * @param string $html
     */
    public function keep(string $html = '.')
    {
        if ($this->flushedTs < time()) {
            // １秒でも経過していたらフラッシュする
            $this->flush($html);
        }
    }

    /**
     * フラッシュする
     *
     * @param string|null $html
     */
    public function flush(string $html = null)
    {
        if ($html === null) {
            return;
        }
        // フラッシュ時間をセットする
        $this->flushedTs = time();

        // サーバー・ブラウザでのバッファリングを防ぐためにレスポンスサイズを大きくする
        $html .= str_pad(' ', 4096);
        echo $html;
        ob_flush();
    }
}
