<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site\Controllers;

use MicroEngine\Cms\Models\Page;
use MicroEngine\Cms\Services\Query;
use Slim\Exception\NotFoundException;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Controllers\Action;
use Psr\Http\Message\ServerRequestInterface;

class PaginationAction extends Action
{
    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @param  Page|null $page
     * @return ResponseInterface
     * @throws NotFoundException
     */
    public function __invoke(ServerRequestInterface $request, ResponseInterface $response, array $args, Page $page = null): ResponseInterface
    {
        $pageNo = $args['page_no'] ?? 1;

        // ルート名・タイプを取得
        $routeName = $request->getAttribute('route')->getName();
        if ($routeName !== null) {
            $type = $routeName;
        } else {
            // RoutingActionから転送された場合
            $type = $page->type;
            $routeName = $type;
        }
        // 設定を取得
        $config = config('pages.' . $type);

        // クエリ作成
        $query = new Query($this->c);
        $pagination = $query->pagination($config['options']['query'], $pageNo, $config['options']);

        // ルート情報をセット
        $pagination->setRouteInfo($this->c, $routeName);

        if ($pagination->outOfBounds()) {
            throw new NotFoundException($request, $response);
        }

        // 何ページ目を表示する場合でも、１ページ目の情報をDBから取得する
        if (!isset($page)) {
            $page = Page::where('type', $type)->first();
            if (!$page) {
                throw new NotFoundException($request, $response);
            }
        }
        // １ページ目のパスをセット
        $pagination->setFirstPath($page->path);
        $this->c->site->pushContent($page, true);

        return $this->c->view->render(
            $response,
            $page->getTemplatePath(),
            compact('pagination', 'page')
        );
    }
}
