<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site\Views;

use Twig_Filter;
use Twig_Function;
use Twig_Extension;
use MicroEngine\Cms\Models\Page;
use MicroEngine\Cms\Models\Table;
use MicroEngine\Cms\Services\Query;
use MicroEngine\Site\Services\Site;
use Twig_Extension_GlobalsInterface;
use MicroEngine\Cms\Models\Category;
use Interop\Container\ContainerInterface;

class SiteExtension extends Twig_Extension implements Twig_Extension_GlobalsInterface
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var Site
     */
    protected $site;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->site = $c->site;
    }

    /**
     * 関数登録
     *
     * @return array
     */
    public function getFunctions(): array
    {
        /* @var Site $this ->site */
        return [
            new Twig_Function('collection', function ($content) {
                $html = $this->site->collection($content);
                return $html;
            }, ['is_safe' => ['html']]),
            new Twig_Function('module', function ($type) {
                $html = $this->site->module($type);
                return $html;
            }, ['is_safe' => ['html']]),
            new Twig_Function('monthly_calendar', function ($type, $date = null, $months = 1) {
                $html = $this->c->calendarManager->monthlyCalendar($type, $date, $months);
                return $html;
            }, ['is_safe' => ['html']]),
            new Twig_Function('monthly_calendar_data', function ($type, $date = null, $months = 1) {
                return $this->c->calendarManager->monthlyCalendarData($type, $date, $months);
            }),
            new Twig_Function('calendar', function ($content) {
                $html = $this->site->calendar($content);
                return $html;
            }, ['is_safe' => ['html']]),
            new Twig_Function('query', function ($key, $params = []) {
                $query = new Query($this->c);
                return $query->query($key, $params);
            }),
            new Twig_Function('latest', function ($type, $limit = 3, $baseType = null) {
                return $this->site->latest($type, $limit, $baseType);
            }),
            new Twig_Function('is_preview', function () {
                return $this->site->isPreview();
            }),
            new Twig_Function('set_field', function ($field, $parentField = null, $index = null) {
                return $this->site->setData($field, $parentField, $index);
            }),
            new Twig_Function('field', function ($field, $content = null) {
                return $this->site->field($field, $content);
            }, ['is_safe' => ['html']]),
            new Twig_Function('field_attr', function ($field, $defaultClass = null) {
                return $this->site->fieldAttr($field, $defaultClass);
            }, ['is_safe' => ['html']]),
            new Twig_Function('child_field', function ($field, $index = null, $parentField = null) {
                return $this->site->childField($field, $index, $parentField);
            }, ['is_safe' => ['html']]),
            new Twig_Function('input_id', function ($input) {
                return $this->site->inputId($input);
            }),
            new Twig_Function('input_name', function ($input, $multiple = false) {
                return $this->site->inputName($input, $multiple);
            }),
            new Twig_Function('field_data_id', function ($input) {
                return $this->site->fieldDataId($input);
            }, ['is_safe' => ['html']]),
            new Twig_Function('page_data_id', function ($id) {
                return $this->site->pageDataId($id);
            }, ['is_safe' => ['html']]),
            new Twig_Function('get_options', function ($input = '', $config = []) {
                return $this->site->getOptions($input, $config);
            }),
            new Twig_Function('data', function ($path, $options = []) {
                $data = $this->c->data;

                return $data->get($path, $options);
            }),
            new Twig_Function('category_id', function ($dir = null) {
                return Category::where('dir', $dir)->value('id');
            }),
            new Twig_Function('pages_in_category', function ($categoryId = null) {
                return Page::where('category_id', $categoryId)->orderBy('display_order')->get();
            }),
            new Twig_Function('breadcrumbs', function ($categoryId = null) {
                return $this->site->breadcrumbs($categoryId);
            }),
            new Twig_Function('sitemap', function () {
                return $this->site->sitemap();
            }),
            new Twig_Function('is_current', function (string $path, string $class = null) {
                return $this->site->isCurrent($path, $class);
            }),
            new Twig_Function('is_current_dir', function (string $path, string $class = null) {
                return $this->site->isCurrentDir($path, $class);
            }),
            new Twig_Function('is_parent', function (string $path, string $class = null) {
                return $this->site->isParent($path, $class);
            }),
        ];
    }

    /**
     * フィルター登録
     *
     * @return array
     */
    public function getFilters(): array
    {
        return [
            new Twig_Filter('resize', function ($path, $option) {
                return resize($path, $option);
            }),
        ];
    }

    /**
     * Twigのグローバル変数登録
     *
     * @return array
     */
    public function getGlobals(): array
    {
        // サイト設定を取得
        try {
            $site = Table::where('type', 'site')->value('data');
        } catch (\Exception $e) {
            // インストール前は例外が発生する
            $site = [];
        }
        if ($site === null) {
            $site = [];
        }
        return [
            '_site' => $site,
        ];
    }
}
