/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

(function ($, window) {
  'use strict';

  if (typeof window.mecms === 'undefined') {
    window.mecms = {};
  }

  /**
   * File Manager
   */
  mecms.fileManager = {
    isModal: false,
    init: function () {
      var self = this, $selectModal, $dropArea, files;

      $('#selectModal').on('click', '[data-create-dir]', function (e) {
        self.createDir(e);
      });
      $('#bulkCheckbox').on('change', function (e) {
        self.bulkCheckbox(e);
      });
      $('#buldDownload').on('click', function (e) {
        self.bulkDownload(e);
      });
      $('#buldDelete').on('click', function (e) {
        self.bulkDelete(e);
      });
      $(document).on('click', '[data-preview]', function (e) {
        self.preview(e);
      });
      $('#previewModal, #dirModal').on('hidden.bs.modal', function (e) {
        $selectModal = $('#selectModal');
        $selectModal.modal('show');
      });
      $dropArea = $('#uploadsModal');
      $dropArea.on('dragenter dragover', function (e) {
        e.preventDefault();
        e.stopPropagation();
        return false;
      });
      $dropArea.on('drop', function (e) {
        e.preventDefault();
        files = e.originalEvent.dataTransfer.files;
        self.bulkUpload(files);
        return false;
      });
      $('#uploadFiles').on('change', function () {
        files = this.files;
        self.bulkUpload(files);
        return false;
      });
      $('#uploadsModal').on('show.bs.modal', function (e) {
        $('#selectModal').modal('hide');
      });
      $('#uploadsModal').on('hidden.bs.modal', function (e) {
        self.closeUploader();
      });
    },
    createDir: function (e) {
      var self = this, $modal, $target, dir;

      $('#selectModal').modal('hide');
      $modal = $('#dirModal');
      $modal.empty().modal('show');
      $target = $(e.currentTarget);
      dir = $target.attr('data-create-dir');

      $.ajax({
        url: '/admin/file/modal/dir/create',
        type: 'GET',
        data: {'dir': dir},
        cache: false
      }).done(function (html) {
        $modal.html(html);
      }).fail(function (jqXHR, textStatus, errorThrown) {
        $modal.html(jqXHR.responseText);
      });
    },
    storeDir: function () {
      var self = this, $modal, $form, dir;

      $modal = $('#dirModal');
      $form = $('#createDirForm');
      dir = $modal.find('[name="dir"]').val() + $modal.find('[name="filename"]').val() + '/';

      $.ajax({
        url: $form.attr('action'),
        type: 'POST',
        data: $form.serialize(),
        cache: false
      }).done(function (html) {
        if (self.isModal) {
          $('#selectModalContent').empty();
          $modal.modal('hide');
          mecms.content.openFileModal(dir);
        } else {
          location.href = '/admin/file/manager?dir=' + dir;
        }
      }).fail(function (jqXHR, textStatus, errorThrown) {
        $modal.html(jqXHR.responseText);
      });
    },
    preview: function (e) {
      var self = this, $target, $modal, $content, $row;

      $target = $(e.currentTarget);
      $row = $target.parents('tr');
      $modal = $('#previewModal');

      $content = $('<img class="img-fluid">');
      $content.attr({
        'src': $target.attr('href')
      });
      $modal.find('.modal-body').empty().append($content);

      $modal.find('#data-filename').html($row.find('[data-filename]').html());
      $modal.find('#data-label').html($row.find('[data-label]').html());
      $modal.find('#data-filesize').html($row.find('[data-filesize]').html());

      $('#selectModal').modal('hide');
      $('#previewModal').modal('show');
      e.preventDefault();

      return false;
    },
    bulkUpload: function (files) {
      var self = this, $resultTbody, fd, $tr, $form;

      $resultTbody = $('#uploadResult');
      $form = $('#uploadsForm');

      fd = new FormData();
      fd.append('csrf_name', $form.find('[name="csrf_name"]').val());
      fd.append('csrf_value', $form.find('[name="csrf_value"]').val());
      fd.append('dir', $('#dir').val());
      for (var i = 0; i < files.length; i++) {
        fd.append('files[]', files[i]);
      }
      $.ajax({
        url: $form.attr('action'),
        type: 'POST',
        data: fd,
        dataType: 'json',
        processData: false,
        contentType: false
      }).done(function (json) {
        $.each(json, function (i, val) {
          $tr = $('<tr><td></td><td></td></tr>');
          $tr.find('td:eq(0)').html(val.filename);
          if (val['class'] == 'success') {
            $tr.find('td:eq(1)').html('<span class="badge badge-pill badge-success">' + val.message + '</span>');
          } else if (val['class'] == 'primary') {
            $tr.find('td:eq(1)').html('<span class="badge badge-pill badge-primary">' + val.message + '</span>');
          } else {
            $tr.find('td:eq(1)').html(val.message);
          }
          $resultTbody.append($tr);
        });
      }).fail(function (jqXHR, textStatus, errorThrown) {
        $tr = $('<tr><td colspan="2"></td></tr>');
        $tr.find('td').html(jqXHR.responseText);
      });
    },
    closeUploader: function () {
      var self = this, $result, dir;

      dir = $('#dir').val();

      $result = $('#uploadResult');
      if (self.isModal) {
        $result.empty();
        mecms.content.openFileModal(dir);
      } else if ($result.children().length > 0) {
        location.reload(true);
      }
    },
    bulkCheckbox: function (e) {
      $('#fileListForm input[type="checkbox"]').prop('checked', $(e.target).is(':checked'));
    },
    bulkDownload: function (e) {
      if ($('input[name="paths[]"]:checked').length < 1) {
        e.preventDefault();
        alert('ファイルが選択されていません。');
        return false;
      }
    },
    bulkDelete: function (e) {
      var self = this, $modal;

      if ($('#fileListForm input[name="paths[]"]:checked').length < 1) {
        e.preventDefault();
        alert('ファイルが選択されていません。');
        return false;
      }

      $modal = $('#destroyModal');
      $modal.modal('show');

      var $inputs = $('#fileListForm input[name="paths[]"]');
      var $form = $('#destroyForm');
      $form.find('input[name="paths[]"]').remove();
      $form.append($inputs.clone().hide());

      return true;
    }
  };

})(jQuery, window);
